@@ -1,8 +1,8 @@
 /* s390.c - core analysis suite
  *
  * Copyright (C) 2001, 2002 Mission Critical Linux, Inc.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2009, 2010 David Anderson
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2009, 2010 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2009, 2010, 2012 David Anderson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2009, 2010, 2012 Red Hat, Inc. All rights reserved.
  * Copyright (C) 2005, 2006, 2010 Michael Holzheu, IBM Corporation
  *
  * This program is free software; you can redistribute it and/or modify
@@ -67,16 +67,25 @@ static void s390_dump_line_number(ulong);
 static struct line_number_hook s390_line_number_hooks[];
 static int s390_is_uvaddr(ulong, struct task_context *);
 
+/*
+ * struct lowcore name (old: "_lowcore", new: "lowcore")
+ */
+static char *lc_struct;
+
 /*
  * Initialize member offsets
  */
 static void s390_offsets_init(void)
 {
-	if (MEMBER_EXISTS("_lowcore", "st_status_fixed_logout"))
-		MEMBER_OFFSET_INIT(s390_lowcore_psw_save_area, "_lowcore",
+	if (STRUCT_EXISTS("lowcore"))
+		lc_struct = "lowcore";
+	else
+		lc_struct = "_lowcore";
+	if (MEMBER_EXISTS(lc_struct, "st_status_fixed_logout"))
+		MEMBER_OFFSET_INIT(s390_lowcore_psw_save_area, lc_struct,
 				   "st_status_fixed_logout");
 	else
-		MEMBER_OFFSET_INIT(s390_lowcore_psw_save_area, "_lowcore",
+		MEMBER_OFFSET_INIT(s390_lowcore_psw_save_area, lc_struct,
 				   "psw_save_area");
 }
 
@@ -578,9 +587,9 @@ static void s390_get_int_stack(char *stack_name, char* lc, char* int_stack,
 {
 	unsigned long stack_addr;
 
-	if (!MEMBER_EXISTS("_lowcore", stack_name))
+	if (!MEMBER_EXISTS(lc_struct, stack_name))
 		return;
-	stack_addr = ULONG(lc + MEMBER_OFFSET("_lowcore", stack_name));
+	stack_addr = ULONG(lc + MEMBER_OFFSET(lc_struct, stack_name));
 	if (stack_addr == 0)
 		return;
 	readmem(stack_addr - INT_STACK_SIZE, KVADDR, int_stack,
@@ -665,6 +674,9 @@ s390_back_trace_cmd(struct bt_info *bt)
 		unsigned long r14_stack_off;
 		struct load_module *lm;
 		int j;
+		ulong offset;
+		char *name_plus_offset;
+		struct syment *sp;
 
 		/* Find stack: Either async, panic stack or task stack */
 		if((backchain > stack_start) && (backchain < stack_end)){
@@ -698,7 +710,15 @@ s390_back_trace_cmd(struct bt_info *bt)
 			skip_first_frame=0;
 		} else {
 			fprintf(fp," #%i [%08lx] ",i,backchain);
-			fprintf(fp,"%s at %x", closest_symbol(r14), r14);
+			name_plus_offset = NULL;
+			if (bt->flags & BT_SYMBOL_OFFSET) {
+				sp = value_search(r14, &offset);
+				if (sp && offset)
+					name_plus_offset = 
+						value_to_symstr(r14, buf, bt->radix);
+			}
+			fprintf(fp,"%s at %x", name_plus_offset ? 
+				name_plus_offset : closest_symbol(r14), r14);
 			if (module_symbol(r14, NULL, &lm, NULL, 0))
 				fprintf(fp, " [%s]", lm->mod_name);
 			fprintf(fp, "\n");
@@ -782,18 +802,18 @@ s390_print_lowcore(char* lc, struct bt_info *bt, int show_symbols)
 		if (bt->flags & BT_LINE_NUMBERS)
 	       		s390_dump_line_number(tmp[1] & S390_ADDR_MASK);
 	}	
-	ptr = lc + MEMBER_OFFSET("_lowcore","cpu_timer_save_area");
+	ptr = lc + MEMBER_OFFSET(lc_struct, "cpu_timer_save_area");
 	tmp[0]=UINT(ptr);
 	tmp[1]=UINT(ptr + S390_WORD_SIZE);
 	fprintf(fp,"  -cpu timer: %#010lx %#010lx\n", tmp[0],tmp[1]);
 
-	ptr = lc + MEMBER_OFFSET("_lowcore","clock_comp_save_area");
+	ptr = lc + MEMBER_OFFSET(lc_struct, "clock_comp_save_area");
 	tmp[0]=UINT(ptr);
 	tmp[1]=UINT(ptr + S390_WORD_SIZE);
 	fprintf(fp,"  -clock cmp: %#010lx %#010lx\n", tmp[0], tmp[1]);
 
 	fprintf(fp,"  -general registers:\n");
-	ptr = lc + MEMBER_OFFSET("_lowcore","gpregs_save_area");
+	ptr = lc + MEMBER_OFFSET(lc_struct, "gpregs_save_area");
 	tmp[0]=ULONG(ptr);
 	tmp[1]=ULONG(ptr + S390_WORD_SIZE);
 	tmp[2]=ULONG(ptr + 2 * S390_WORD_SIZE);
@@ -820,7 +840,7 @@ s390_print_lowcore(char* lc, struct bt_info *bt, int show_symbols)
 		tmp[0], tmp[1], tmp[2], tmp[3]);
 
 	fprintf(fp,"  -access registers:\n");
-	ptr = lc + MEMBER_OFFSET("_lowcore","access_regs_save_area");
+	ptr = lc + MEMBER_OFFSET(lc_struct, "access_regs_save_area");
 	tmp[0]=ULONG(ptr);
 	tmp[1]=ULONG(ptr + S390_WORD_SIZE);
 	tmp[2]=ULONG(ptr + 2 * S390_WORD_SIZE);
@@ -847,7 +867,7 @@ s390_print_lowcore(char* lc, struct bt_info *bt, int show_symbols)
 		tmp[0], tmp[1], tmp[2], tmp[3]);
 
 	fprintf(fp,"  -control registers:\n");
-	ptr = lc + MEMBER_OFFSET("_lowcore","cregs_save_area");
+	ptr = lc + MEMBER_OFFSET(lc_struct, "cregs_save_area");
 	tmp[0]=ULONG(ptr);
 	tmp[1]=ULONG(ptr + S390_WORD_SIZE);
 	tmp[2]=ULONG(ptr + 2 * S390_WORD_SIZE);
@@ -874,7 +894,7 @@ s390_print_lowcore(char* lc, struct bt_info *bt, int show_symbols)
 	fprintf(fp,"     %#010lx %#010lx %#010lx %#010lx\n", 
 		tmp[0], tmp[1], tmp[2], tmp[3]);
 
-	ptr = lc + MEMBER_OFFSET("_lowcore","floating_pt_save_area");
+	ptr = lc + MEMBER_OFFSET(lc_struct, "floating_pt_save_area");
 	fprintf(fp,"  -floating point registers 0,2,4,6:\n");
 	tmp[0]=ULONG(ptr);
 	tmp[1]=ULONG(ptr + 2 * S390_WORD_SIZE);
@@ -900,7 +920,7 @@ s390_get_stack_frame(struct bt_info *bt, ulong *eip, ulong *esp)
 	/* get the stack pointer */
 	if(esp){
 		if(s390_has_cpu(bt)){
-			ksp = ULONG(lowcore + MEMBER_OFFSET("_lowcore",
+			ksp = ULONG(lowcore + MEMBER_OFFSET(lc_struct,
 				"gpregs_save_area") + (15 * S390_WORD_SIZE));
 		} else {
 			readmem(bt->task + OFFSET(task_struct_thread_ksp),
